==> spread-5.5.0.34-py3-none-any.whl: spread_core/tests/equipment/light/test_engineries.py <==
from unittest import TestCase

from spread_core.mqtt.spread.address import ProviderAddress

from spread_core.bam.engineries import light
from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable
from spread_core.tests.equipment.utils import TestEquipmentAdapter, TestEngineryRecipe
from spread_core.bam import const as C 


class _TestLight(TestCase):
    enginery_type = light.Light

    @classmethod
    def setUpClass(cls) -> None:
        cls.adapter = TestEquipmentAdapter()
        cls.recipe = TestEngineryRecipe()

    def setUp(self) -> None:
        enginery = self.enginery_type(999, power=99)
        enginery.adapter = self.adapter
        enginery._recipe = self.recipe

        self.enginery = enginery

    def _emulate_state(self, funit, value):
        self.enginery.on_state(State(ProviderAddress(1, 'Manager', 2, 'Provider', 3, funit)), Variable(value))

    def _check_switching_detection(self, value_of_state, expected_value):
        self.assertEqual(self.enginery.calc_on(Variable(value_of_state)), expected_value)

    def _check_value(self, funit, expected_value):
        self.assertEqual(self.enginery.values.get(funit), expected_value)

    def _check_end_of_publishes(self):
        if self.adapter.queue:
            unexpected_publishes = f'Unexpected publishes ({len(self.adapter.queue)}):\n'
            unexpected_publishes += "\n".join(str(i) for i in self.adapter.queue)
            self.fail(unexpected_publishes)

    def _check_publish(self, _topic, _value, _retain=True, _delay=0):
        topic, variable, retain, delay = self.adapter.queue.popleft()
        self.assertEqual(topic, _topic)
        self.assertEqual(variable.value, _value)
        self.assertEqual(retain, _retain)
        self.assertEqual(delay, _delay)

    def test_recipe_appointment(self):
        recipe = TestEngineryRecipe(1001)
        self.enginery.set_recipe('SomeString', recipe)
        self.assertIs(self.enginery.recipe, recipe)

    def test_switching_detector(self):
        self._check_switching_detection(None, None)
        self._check_switching_detection(0, False)
        self._check_switching_detection(1, True)

    def test_address(self):
        self.assertEqual(self.enginery.get_address('SomeFunit'), f'1/Equipment/{self.enginery_type.__name__}/999/SomeFunit')

    def get_address_broadcast(self):
        self.assertEqual(self.enginery.get_address_broadcast('SomeFunit'), f'1/Equipment/{self.enginery_type.__name__}/+/SomeFunit')

    def test_recipe_selecting(self):
        funit, recipe = self.enginery.recipe_for_funit('SomeFunit')
        self.assertEqual(funit, 'SomeFunit')
        self.assertEqual(recipe, self.recipe)

    def test_state_handler(self):
        self._emulate_state('BrightnessLevel', 100)

        self._check_value('BrightnessLevel', 100)
        self._check_value('On', True)
        self._check_value('Power', 99)

        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/BrightnessLevel', 100)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/On', True)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Power', 99)

        self._check_end_of_publishes()

    def test_invalidation(self):
        self.enginery.values['Funit1'] = 1
        self.enginery.values['Funit2'] = 'Funit2'
        self.enginery.values['Funit3'] = [1, 2, 3]

        self.enginery.set_invalid()

        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Funit1', None)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Funit2', None)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Funit3', None)

        self._check_end_of_publishes()

    def test_funit_On(self):
        funit = light.On(self.enginery, Variable(True))
        funit.set()
        self._check_publish('Spread/Set/Provider/1/BrightnessLevel', 100, False, 0)
        self._check_end_of_publishes()

    def test_funit_BrightnessLevel(self):
        funit = light.BrightnessLevel(self.enginery, Variable(100))
        funit.set()
        self._check_publish('Spread/Set/Provider/1/BrightnessLevel', 100, False, 0)
        self._check_end_of_publishes()

    def test_funit_BrightnessLevel_invalid(self):
        self.assertRaises(ValueError, light.BrightnessLevel(self.enginery, Variable('asd')).set)
        self.assertRaises(ValueError, light.BrightnessLevel(self.enginery, Variable(-1)).set)
        self.assertRaises(ValueError, light.BrightnessLevel(self.enginery, Variable(101)).set)

    def test_funit_Power(self):
        funit = light.Power(self.enginery, Variable(100))
        self.assertRaises(light.Power.NotImplemented, funit.set)

    def test_funit_GroupLevel(self):
        funit = light.GroupLevel(self.enginery, Variable(100))
        funit.set()
        self._check_publish('Spread/Set/Provider/1/GroupLevel', 100, False, 0)
        self._check_end_of_publishes()

    def test_funit_GroupLevel_invalid(self):
        self.assertRaises(ValueError, light.GroupLevel(self.enginery, Variable('asd')).set)
        self.assertRaises(ValueError, light.GroupLevel(self.enginery, Variable(-1)).set)
        self.assertRaises(ValueError, light.GroupLevel(self.enginery, Variable(101)).set)

    def test_funit_GroupOn(self):
        funit = light.GroupOn(self.enginery, Variable(True))
        funit.set()
        self._check_publish('Spread/Set/Provider/1/GroupLevel', 100, False, 0)
        self._check_end_of_publishes()


class TestDimmer(_TestLight):
    enginery_type = light.DimmingLight


class TestTW(_TestLight):
    enginery_type = light.TunableWhiteLight

    def test_state_handler(self):
        super(TestTW, self).test_state_handler()

        self._emulate_state('Temperature', 100)
        self._check_value('Temperature', 100)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Temperature', 100)
        self._check_end_of_publishes()

    def test_funit_Temperature(self):
        funit = light.Temperature(self.enginery, Variable(1234))
        funit.set()
        self._check_publish('Spread/Set/Provider/1/Temperature', 1234, False, 0)
        self._check_end_of_publishes()

    def test_funit_Temperature_invalid(self):
        self.assertRaises(ValueError, light.Temperature(self.enginery, Variable('asd')).set)
        self.assertRaises(ValueError, light.Temperature(self.enginery, Variable(999)).set)
        self.assertRaises(ValueError, light.Temperature(self.enginery, Variable(20001)).set)


COLOR_11 = 433
COLOR_22 = 866
COLOR_33 = 1299
COLOR_44 = 1732
COLOR_128 = 5039
COLOR_254 = 10000
COLOR_255 = None


class TestRgb(_TestLight):
    enginery_type = light.RgbLight
    value = {
        C.RED: COLOR_11,
        C.GREEN: COLOR_22,
        C.BLUE: COLOR_33,
    }

    def test_state_handler(self):
        super(TestRgb, self).test_state_handler()

        self._emulate_state('Color', {**self.value, 'someElse': COLOR_128})
        self._check_value('Color', self.value)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Color', self.value)
        self._check_end_of_publishes()

        self._emulate_state('Color', {C.RED: COLOR_255, C.GREEN: COLOR_255})

        self._check_value('Color', None)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Color', None)
        self._check_end_of_publishes()

    def test_funit_Color(self):
        funit = light.Color(self.enginery, Variable(self.value))
        funit.set()
        self._check_publish('Spread/Set/Provider/1/Color', self.value, False, 0)
        self._check_end_of_publishes()

    def test_funit_Color_invalid(self):
        self.assertRaises(ValueError, light.Color(self.enginery, Variable({**self.value, C.RED: COLOR_254+1})).set)
        self.assertRaises(ValueError, light.Color(self.enginery, Variable({**self.value, 'Unknown': COLOR_128})).set)
        self.assertRaises(ValueError, light.Color(self.enginery, Variable(999)).set)


class TestRgbw(TestRgb):
    enginery_type = light.RgbwLight
    value = {
        C.RED: COLOR_11,
        C.GREEN: COLOR_22,
        C.BLUE: COLOR_33,
        C.WHITE: COLOR_44
    }


class TestDynamic(_TestLight):
    enginery_type = light.DynamicLight


class TestRelay(_TestLight):
    enginery_type = light.SwitchingLight

    def test_switching_detector(self):
        self._check_switching_detection(None, None)
        self._check_switching_detection(99, False)
        self._check_switching_detection(100, True)

==> spread-5.8.1.80-py3-none-any.whl: spread_core/tests/equipment/light/test_engineries.py <==
from unittest import TestCase

from spread_core.mqtt.spread.address import ProviderAddress

from spread_core.bam.engineries import light
from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable
from spread_core.tests.equipment.utils import TestEquipmentAdapter, TestEngineryRecipe
from spread_core.bam import const as C 


class _TestLight(TestCase):
    enginery_type = light.Light

    @classmethod
    def setUpClass(cls) -> None:
        cls.adapter = TestEquipmentAdapter()
        cls.recipe = TestEngineryRecipe()

    def setUp(self) -> None:
        enginery = self.enginery_type(999, power=99)
        enginery.adapter = self.adapter
        enginery._recipe = self.recipe

        self.enginery = enginery

    def _emulate_state(self, funit, value):
        self.enginery.on_state(State(ProviderAddress(1, 'Manager', 2, 'Provider', 3, funit)), Variable(value))

    def _check_switching_detection(self, value_of_state, expected_value):
        self.assertEqual(self.enginery.calc_on(Variable(value_of_state)), expected_value)

    def _check_value(self, funit, expected_value):
        self.assertEqual(self.enginery.values.get(funit), expected_value)

    def _check_end_of_publishes(self):
        if self.adapter.queue:
            unexpected_publishes = f'Unexpected publishes ({len(self.adapter.queue)}):\n'
            unexpected_publishes += "\n".join(str(i) for i in self.adapter.queue)
            self.fail(unexpected_publishes)

    def _check_publish(self, _topic, _value, _retain=True, _delay=0):
        topic, variable, retain, delay = self.adapter.queue.popleft()
        self.assertEqual(topic, _topic)
        self.assertEqual(variable.value, _value)
        self.assertEqual(retain, _retain)
        self.assertEqual(delay, _delay)

    def test_recipe_appointment(self):
        recipe = TestEngineryRecipe(1001)
        self.enginery.set_recipe('SomeString', recipe)
        self.assertIs(self.enginery.recipe, recipe)

    def test_switching_detector(self):
        self._check_switching_detection(None, None)
        self._check_switching_detection(0, False)
        self._check_switching_detection(1, True)

    def test_address(self):
        self.assertEqual(self.enginery.get_address('SomeFunit'), f'1/Equipment/{self.enginery_type.__name__}/999/SomeFunit')

    def get_address_broadcast(self):
        self.assertEqual(self.enginery.get_address_broadcast('SomeFunit'), f'1/Equipment/{self.enginery_type.__name__}/+/SomeFunit')

    def test_recipe_selecting(self):
        funit, recipe = self.enginery.recipe_for_funit('SomeFunit')
        self.assertEqual(funit, 'SomeFunit')
        self.assertEqual(recipe, self.recipe)

    def test_state_handler(self):
        self._emulate_state('BrightnessLevel', 100)

        self._check_value('BrightnessLevel', 100)
        self._check_value('On', True)
        self._check_value('Power', 99)

        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/BrightnessLevel', 100)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/On', True)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Power', 99)

        self._check_end_of_publishes()

    def test_invalidation(self):
        self.enginery.values['Funit1'] = 1
        self.enginery.values['Funit2'] = 'Funit2'
        self.enginery.values['Funit3'] = [1, 2, 3]

        self.enginery.set_invalid()

        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Funit1', None)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Funit2', None)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Funit3', None)

        self._check_end_of_publishes()

    def test_funit_On(self):
        funit = light.On(self.enginery, Variable(True))
        funit.set()
        self._check_publish('Spread/Set/Provider/1/BrightnessLevel', 100, False, 0)
        self._check_end_of_publishes()

    def test_funit_BrightnessLevel(self):
        funit = light.BrightnessLevel(self.enginery, Variable(100))
        funit.set()
        self._check_publish('Spread/Set/Provider/1/BrightnessLevel', 100, False, 0)
        self._check_end_of_publishes()

    def test_funit_BrightnessLevel_invalid(self):
        self.assertRaises(ValueError, light.BrightnessLevel(self.enginery, Variable('asd')).set)
        self.assertRaises(ValueError, light.BrightnessLevel(self.enginery, Variable(-1)).set)
        self.assertRaises(ValueError, light.BrightnessLevel(self.enginery, Variable(101)).set)

    def test_funit_Power(self):
        funit = light.Power(self.enginery, Variable(100))
        self.assertRaises(light.Power.NotImplemented, funit.set)

    def test_funit_GroupLevel(self):
        funit = light.GroupLevel(self.enginery, Variable(100))
        funit.set()
        self._check_publish('Spread/Set/Provider/1/GroupLevel', 100, False, 0)
        self._check_end_of_publishes()

    def test_funit_GroupLevel_invalid(self):
        self.assertRaises(ValueError, light.GroupLevel(self.enginery, Variable('asd')).set)
        self.assertRaises(ValueError, light.GroupLevel(self.enginery, Variable(-1)).set)
        self.assertRaises(ValueError, light.GroupLevel(self.enginery, Variable(101)).set)

    def test_funit_GroupOn(self):
        funit = light.GroupOn(self.enginery, Variable(True))
        funit.set()
        self._check_publish('Spread/Set/Provider/1/GroupLevel', 100, False, 0)
        self._check_end_of_publishes()


class TestDimmer(_TestLight):
    enginery_type = light.DimmingLight


class TestTW(_TestLight):
    enginery_type = light.TunableWhiteLight

    def test_state_handler(self):
        super(TestTW, self).test_state_handler()

        self._emulate_state('Temperature', 100)
        self._check_value('Temperature', 100)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Temperature', 100)
        self._check_end_of_publishes()

    def test_funit_Temperature(self):
        funit = light.Temperature(self.enginery, Variable(1234))
        funit.set()
        self._check_publish('Spread/Set/Provider/1/Temperature', 1234, False, 0)
        self._check_end_of_publishes()

    def test_funit_Temperature_invalid(self):
        self.assertRaises(ValueError, light.Temperature(self.enginery, Variable('asd')).set)
        self.assertRaises(ValueError, light.Temperature(self.enginery, Variable(999)).set)
        self.assertRaises(ValueError, light.Temperature(self.enginery, Variable(20001)).set)


COLOR_11 = 433
COLOR_22 = 866
COLOR_33 = 1299
COLOR_44 = 1732
COLOR_128 = 5039
COLOR_254 = 10000
COLOR_255 = None


class TestRgb(_TestLight):
    enginery_type = light.RgbLight
    value = {
        C.RED: COLOR_11,
        C.GREEN: COLOR_22,
        C.BLUE: COLOR_33,
    }

    def test_state_handler(self):
        super(TestRgb, self).test_state_handler()

        self._emulate_state('Color', {**self.value, 'someElse': COLOR_128})
        self._check_value('Color', self.value)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Color', self.value)
        self._check_end_of_publishes()

        self._emulate_state('Color', {C.RED: COLOR_255, C.GREEN: COLOR_255})

        self._check_value('Color', None)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Color', None)
        self._check_end_of_publishes()

    def test_funit_Color(self):
        funit = light.Color(self.enginery, Variable(self.value))
        funit.set()
        self._check_publish('Spread/Set/Provider/1/Color', self.value, False, 0)
        self._check_end_of_publishes()

    def test_funit_Color_invalid(self):
        self.assertRaises(ValueError, light.Color(self.enginery, Variable({**self.value, C.RED: COLOR_254+1})).set)
        self.assertRaises(ValueError, light.Color(self.enginery, Variable({**self.value, 'Unknown': COLOR_128})).set)
        self.assertRaises(ValueError, light.Color(self.enginery, Variable(999)).set)


class TestRgbw(TestRgb):
    enginery_type = light.RgbwLight
    value = {
        C.RED: COLOR_11,
        C.GREEN: COLOR_22,
        C.BLUE: COLOR_33,
        C.WHITE: COLOR_44
    }


class TestDynamic(_TestLight):
    enginery_type = light.DynamicLight


class TestRelay(_TestLight):
    enginery_type = light.SwitchingLight

    def test_switching_detector(self):
        self._check_switching_detection(None, None)
        self._check_switching_detection(False, False)
        self._check_switching_detection(True, True)

    def test_state_handler(self):
        self._emulate_state('On', True)

        self._check_value('On', True)
        self._check_value('Power', 99)

        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/On', True)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Power', 99)

        self._check_end_of_publishes()

    def test_funit_On(self):
        funit = light.On(self.enginery, Variable(True))
        funit.set()
        self._check_publish('Spread/Set/Provider/1/On', True, False, 0)
        self._check_end_of_publishes()


class TestEmergencyUnit(_TestLight):
    enginery_type = light.EmergencyUnit

    def test_state_handler(self):
        pass

    def test_switching_detector(self):
        pass

    def test_funit_On(self):
        pass

    def _test_battery_level(self, provider_level, enginery_expected):
        self._emulate_state('BatteryLevel', provider_level)
        self._check_value('BatteryLevel', enginery_expected)

        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/BatteryLevel', enginery_expected)

    def test_BatteryLevel(self):
        self._test_battery_level(0, 0)
        self._test_battery_level(100, 100)
        self._test_battery_level(49, 49)

    def test_EmergencyMode(self):
        self._emulate_state('EmergencyMode', 'SomeMode')
        self._check_value('EmergencyMode', 'SomeMode')

        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/EmergencyMode', 'SomeMode')
